/*
  implementation for fetching a raw audio-stream
  Copyright (C) 2000  Martin Vogt

  based on:
  implementation for winskin fft
  Copyright (C) 2000  Martin Vogt

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU Library General Public License as published by
  the Free Software Foundation.

  For more information look at the file COPYRIGHT in this package
 */

#include "rawStream_impl.h"
#include <audiosubsys.h>
#include <convert.h>

namespace Noatun
{

const unsigned int maxBufferSize = 65536;

RawStream_impl::RawStream_impl()
{
}

RawStream_impl::~RawStream_impl()
{
}

void RawStream_impl::streamInit()
{
}

void RawStream_impl::streamStart()
{
}

void RawStream_impl::calculateBlock(unsigned long samples)
{
	int n = sizeof(float) * samples;
	long int pcm_data[samples];

	memcpy(outleft, inleft, n);
	memcpy(outright, inright, n);

	convert_stereo_2float_i16le(samples, inleft, inright, (unsigned char *)pcm_data);

	// Don't let the buffer get out of control
	if (_buffer.size() >= maxBufferSize)
	{
		_buffer.clear();
		return;
	}

	_buffer.reserve(_buffer.size() + samples);
	for (unsigned long x = 0;x < samples;x++)
	{
		_buffer.push_back(pcm_data[x]);
	}
}

vector<long> *RawStream_impl::queuedData()
{
	vector<long> *ret = new vector<long>(_buffer);
	// It's consumed, get rid of it
	_buffer.resize(0);
	return ret;
}

bool RawStream_impl::dataWaiting()
{
	return _buffer.size();
}

void RawStream_impl::dataWaiting(bool)
{
}

REGISTER_IMPLEMENTATION(RawStream_impl);

}; // END namespace Noatun
